#include "main.h"

typedef struct _Smart_Data Smart_Data;

struct _Smart_Data
{ 
   Evas_Coord   x, y, w, h;
   Evas_Object *obj;
   char         fill_inside : 1;
}; 

static void _icon_smart_reconfigure(Smart_Data *sd);
static void _icon_smart_init(void);
static void _icon_smart_add(Evas_Object *obj);
static void _icon_smart_del(Evas_Object *obj);
static void _icon_smart_layer_set(Evas_Object *obj, int layer);
static void _icon_smart_raise(Evas_Object *obj);
static void _icon_smart_lower(Evas_Object *obj);
static void _icon_smart_stack_above(Evas_Object *obj, Evas_Object *above);
static void _icon_smart_stack_below(Evas_Object *obj, Evas_Object *below);
static void _icon_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
static void _icon_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h);
static void _icon_smart_show(Evas_Object *obj);
static void _icon_smart_hide(Evas_Object *obj);
static void _icon_smart_color_set(Evas_Object *obj, int r, int g, int b, int a);
static void _icon_smart_clip_set(Evas_Object *obj, Evas_Object * clip);
static void _icon_smart_clip_unset(Evas_Object *obj);

static Evas_Smart *_smart = NULL;

Evas_Object *
icon_add(Evas *evas)
{
   _icon_smart_init();
   return evas_object_smart_add(evas, _smart);
}

void
icon_file_set(Evas_Object *obj, const char *file)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   /* smart code here */
   evas_object_image_file_set(sd->obj, file, NULL);
   _icon_smart_reconfigure(sd);
}

const char *
icon_file_get(Evas_Object *obj)
{
   Smart_Data *sd;
   char *file;
   
   sd = evas_object_smart_data_get(obj);   
   evas_object_image_file_get(sd->obj, &file, NULL);
   return file;
}

void
icon_smooth_scale_set(Evas_Object *obj, int smooth)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   evas_object_image_smooth_scale_set(sd->obj, smooth);
}

int
icon_smooth_scale_get(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);   
   return evas_object_image_smooth_scale_get(sd->obj);
}

void
icon_size_get(Evas_Object *obj, int *w, int *h)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   return evas_object_image_size_get(sd->obj, w, h);
}

int
icon_fill_inside_get(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (sd->fill_inside) return 1;
   return 0;
}

void
icon_fill_inside_set(Evas_Object *obj, int fill_inside)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (((sd->fill_inside) && (fill_inside)) ||
       ((!sd->fill_inside) && (!fill_inside))) return;
   sd->fill_inside = fill_inside;
   _icon_smart_reconfigure(sd);
}

static void
_icon_smart_reconfigure(Smart_Data *sd)
{
   int iw, ih;
   Evas_Coord x, y, w, h;
   
   ih = 0;
   ih = 0;
   evas_object_image_size_get(sd->obj, &iw, &ih);
   if (iw < 1) iw = 1;
   if (ih < 1) ih = 1;

   if (sd->fill_inside)
     {
	w = sd->w;
	h = ((double)ih * w) / (double)iw;
	if (h > sd->h)
	  {
	     h = sd->h;
	     w = ((double)iw * h) / (double)ih;
	  }
     }
   else
     {
	w = sd->w;
	h = ((double)ih * w) / (double)iw;
	if (h < sd->h)
	  {
	     h = sd->h;
	     w = ((double)iw * h) / (double)ih;
	  }	
     }
   x = sd->x + ((sd->w - w) / 2);
   y = sd->y + ((sd->h - h) / 2);
   evas_object_move(sd->obj, x, y);
   evas_object_image_fill_set(sd->obj, 0, 0, w, h);
   evas_object_resize(sd->obj, w, h);
}

static void
_icon_smart_init(void)
{
   if (_smart) return;
   _smart = evas_smart_new("icon",
			     _icon_smart_add,
			     _icon_smart_del,
			     _icon_smart_layer_set,
			     _icon_smart_raise,
			     _icon_smart_lower,
			     _icon_smart_stack_above,
			     _icon_smart_stack_below,
			     _icon_smart_move,
			     _icon_smart_resize,
			     _icon_smart_show,
			     _icon_smart_hide,
			     _icon_smart_color_set,
			     _icon_smart_clip_set,
			     _icon_smart_clip_unset,
			     NULL);
}

static void
_icon_smart_add(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = calloc(1, sizeof(Smart_Data));
   if (!sd) return;
   sd->obj = evas_object_image_add(evas_object_evas_get(obj));
   sd->x = 0;
   sd->y = 0;
   sd->w = 0;
   sd->h = 0;
   sd->fill_inside = 1;
   evas_object_smart_member_add(sd->obj, obj);
   evas_object_smart_data_set(obj, sd);
}
   
static void
_icon_smart_del(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_del(sd->obj);
   free(sd);
}
   
static void
_icon_smart_layer_set(Evas_Object *obj, int layer)
{
   Smart_Data *sd;
      
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_layer_set(sd->obj, layer);
}

static void
_icon_smart_raise(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_raise(sd->obj);
}

static void
_icon_smart_lower(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return; 
   evas_object_lower(sd->obj);
}
                                                             
static void
_icon_smart_stack_above(Evas_Object *obj, Evas_Object *above)
{
   Smart_Data *sd;

   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_stack_above(sd->obj, above);
}
   
static void
_icon_smart_stack_below(Evas_Object *obj, Evas_Object *below)
{
   Smart_Data *sd;
      
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_stack_below(sd->obj, below);
}

static void
_icon_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   sd->x = x;
   sd->y = y;
   _icon_smart_reconfigure(sd);
}

static void
_icon_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   sd->w = w;
   sd->h = h;
   _icon_smart_reconfigure(sd);
}

static void
_icon_smart_show(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_show(sd->obj);
}

static void
_icon_smart_hide(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_hide(sd->obj);
}

static void
_icon_smart_color_set(Evas_Object *obj, int r, int g, int b, int a)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_color_set(sd->obj, r, g, b, a);
}

static void
_icon_smart_clip_set(Evas_Object *obj, Evas_Object * clip)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_clip_set(sd->obj, clip);
}

static void
_icon_smart_clip_unset(Evas_Object *obj)
{
   Smart_Data *sd;
   
   sd = evas_object_smart_data_get(obj);
   if (!sd) return;
   evas_object_clip_unset(sd->obj);
}  
