#include "main.h"

typedef struct _Sc Sc;
typedef struct _Sc_Item Sc_Item;

struct _Sc
{
   unsigned char deleted : 1;
   unsigned char shown: 1;
   
   Evas_Object *o_sc;
   Evas_Object *o_list;
   Evas_Object *o_icon;
   
   Evas_List *files;
   
   int selection;
};

struct _Sc_Item
{
   char *file;
   char *label;
   Evas_Object *edje;
   Evas_Object *icon;
};

static void _sc_new(void *sc);
static void _sc_free(void *sc);
static void _sc_go(void *sc);
static void _sc_ungo(void *sc);

static void _sc_cb_item_select(Evas_Object *o);
static void _sc_cb_item_unselect(Evas_Object *o);

static void _sc_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _sc_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source);

Sc *cur_sc = NULL;

void
wallpaper_start(void)
{
   Sc *sc;
   
   sc = calloc(1, sizeof(Sc));
   screen_push(_sc_new, _sc_free, _sc_go, _sc_ungo, sc);
   screen_go();
}

/* == -- == */

static void
_sc_new(void *sc)
{
   Evas_List *files, *l;
   
   cur_sc = sc;
   files = file_ls(WP);
   for (l = files; l; l = l->next)
     {
	char buf[4096], *f;
	Sc_Item *si;
	
	f = l->data;
	if (f[0] == '.') continue;
	
	snprintf(buf, sizeof(buf), "%s%s", WP, f);
	if ((!file_is_dir(buf)))
	  {
	     si = calloc(1, sizeof(Sc_Item));
	     si->file = strdup(buf);
	     si->label = file_get_file_no_ext(si->file);
	  }
	cur_sc->files = evas_list_append(cur_sc->files, si);
     }   
}

static void
_sc_free(void *sc)
{
   if (!cur_sc) return;
   cur_sc->deleted = 1;
   cur_sc->shown = 0;
   edje_object_signal_emit(cur_sc->o_sc, "stop", "");
}

static void
_sc_go(void *sc)
{
   Evas_List *l;
   
   if (!cur_sc) return;
   
   cur_sc->shown = 1;

   if (!cur_sc->o_sc)
     {
	Evas_Object *o;
	Evas_Coord w, h;
	
	evas_output_viewport_get(evas, NULL, NULL, &w, &h);
	
	o = edje_object_add(evas);
	edje_object_file_set(o, ED"default.edj", "screen/wallpaper");
	evas_object_move(o, 0, 0);
	evas_object_resize(o, w, h);
	evas_object_event_callback_add(o, EVAS_CALLBACK_KEY_DOWN, _sc_cb_key_down, sc);
	edje_object_signal_callback_add(o, "stop", "done", _sc_cb_stop_done, sc);
	cur_sc->o_sc = o;
	
	o = list_add(evas);
	edje_object_part_swallow(cur_sc->o_sc, "list", o);
	list_select_func_set(o, _sc_cb_item_select, _sc_cb_item_unselect);
	cur_sc->o_list = o;
	
	o = icon_add(evas);
	icon_file_set(o, "");
	icon_fill_inside_set(o, 1);
	edje_object_part_swallow(cur_sc->o_sc, "item", o);
	cur_sc->o_icon = o;
	
	list_freeze(cur_sc->o_list);
	for (l = cur_sc->files; l; l = l->next)
	  {
	     Sc_Item *si;
	     
	     si = l->data;
	     
	     o = edje_object_add(evas);
	     edje_object_file_set(o, ED"default.edj", "screen/wallpaper/item");
	     edje_object_part_text_set(o, "label", si->label);
	     list_pack_info_set(o, 128, 64);
	     list_pack_end(cur_sc->o_list, o);
	     si->edje = o;
	     
	     o = icon_add(evas);
	     icon_file_set(o, si->file);
	     icon_fill_inside_set(o, 1);
	     icon_smooth_scale_set(o, 0);
	     edje_object_part_swallow(si->edje, "item", o);
	     si->icon = o;
	  }
	list_thaw(cur_sc->o_list);
     }
   
   list_select_time_set(cur_sc->o_list, 0.0);
   list_select_set(cur_sc->o_list, cur_sc->selection);
   list_select_set(cur_sc->o_list, cur_sc->selection);
   list_select_time_set(cur_sc->o_list, 0.5);
   
   evas_object_show(cur_sc->o_sc);
   evas_object_show(cur_sc->o_list);
   edje_object_signal_emit(cur_sc->o_sc, "start", "");
   evas_object_focus_set(cur_sc->o_sc, 1);
   
   for (l = cur_sc->files; l; l = l->next)
     {
	Sc_Item *si;
	
	si = l->data;
	evas_object_show(si->edje);
	evas_object_show(si->icon);
     }
}

static void
_sc_ungo(void *sc)
{
   if (!cur_sc) return;
   cur_sc->shown = 0;
   edje_object_signal_emit(cur_sc->o_sc, "stop", "");
}

static void
_sc_cb_item_select(Evas_Object *o)
{
   if (!cur_sc) return;
   if (cur_sc->o_sc)
     {
	Sc_Item *si;
	
	edje_object_signal_emit(o, "select", "item");
	edje_object_signal_emit(cur_sc->o_sc, "select", "item");
	si = evas_list_nth(cur_sc->files, cur_sc->selection);
	if (si) icon_file_set(cur_sc->o_icon, si->file);
     }
}
    
static void
_sc_cb_item_unselect(Evas_Object *o)
{
   if (!cur_sc) return;
   if (cur_sc->o_sc)
     {
	edje_object_signal_emit(o, "unselect", "item");
     }
}
    
/* NB: on ipaq
 *
 * "Escape"
 *
 * "F1"   "F2"    "F3"   "F4"
 *            "Up"
 *   "Left" "Return" "Right"
 *           "Down"
 * 
 */
static void
_sc_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
   Evas_Event_Key_Down *ev;
   
   if (!cur_sc) return;
   ev = event_info;
   if (!strcmp(ev->keyname, "Up"))
     {
	int n;
	
	n = evas_list_count(cur_sc->files);
	cur_sc->selection--;
	if (cur_sc->selection < 0) cur_sc->selection += n;
	if (cur_sc->o_list) list_select_set(cur_sc->o_list, cur_sc->selection);
     }
   else if (!strcmp(ev->keyname, "Down"))
     {
	int n;
	
	n = evas_list_count(cur_sc->files);
	cur_sc->selection++;
	if (cur_sc->selection >= n) cur_sc->selection -= n;
	if (cur_sc->o_list) list_select_set(cur_sc->o_list, cur_sc->selection);
     }
   else if ((!strcmp(ev->keyname, "Left")) ||
	    (!strcmp(ev->keyname, "Escape")))
     {
	screen_pop();
     }
   else if ((!strcmp(ev->keyname, "Right")) ||
	    (!strcmp(ev->keyname, "Return")))
     {
	Sc_Item *si;

	si = evas_list_nth(cur_sc->files, cur_sc->selection);
	if (si)
	  {
	     bg_file_set(si->file);
	  }
     }
}

static void
_sc_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source)
{
   Sc *sc;
   Evas_List *l;
   
   sc = data;
   
   evas_object_hide(cur_sc->o_sc);
   
   if (sc->shown) return;
   
   evas_object_del(sc->o_sc);
   evas_object_del(sc->o_list);
   evas_object_del(sc->o_icon);
   sc->o_sc = NULL;
   sc->o_list = NULL;
   sc->o_icon = NULL;

   for (l = sc->files; l; l = l->next)
     {
	Sc_Item *si;
	
	si = l->data;
	evas_object_del(si->edje);
	evas_object_del(si->icon);
	si->edje = NULL;
	si->icon = NULL;
     }
   
   if (!sc->deleted) return;
   
   while (sc->files)
     {
	Sc_Item *si;
	
	si = sc->files->data;
	sc->files = evas_list_remove(sc->files, si);
	if (si->file) free(si->file);
	if (si->label) free(si->label);
	free(si);
     }
   
   free(sc);
}       
